import os
from .config_strings import *

default_dir = "/tmp/torch-cv-test/config"
os.makedirs(default_dir, exist_ok=True)


def create_empty_config():
    dest = os.path.join(default_dir, "empty.yml")
    with open(dest, "w") as f:
        pass


def create_none_config():
    dest = os.path.join(default_dir, "none.yml")
    with open(dest, "w") as f:
        f.write(none_config)


def create_join_config():
    dest = os.path.join(default_dir, "join.yml")
    with open(dest, "w") as f:
        f.write(join_config)


def create_preprocess_config():
    dest = os.path.join(default_dir, "preprocess.yml")
    with open(dest, "w") as f:
        f.write(preprocess_config)



import os

import pandas as pd
from joblib import dump

from .base import BasePreprocessingClass


class CreateLabelMap(BasePreprocessingClass):
    def __init__(self, src: str, dest: str, target: str = "class"):
        super().__init__()
        self.dest = dest

        self.df = self._read_csv(src)
        self.label_list = self.df[target].unique()

    def __call__(self, *args, **kwargs):
        import time
        start = time.time()

        os.makedirs(self.dest, exist_ok=True)

        label_map = {}
        for i, label in enumerate(self.label_list):
            label_map[label] = i

        dump(label_map, os.path.join(self.dest, "label_map.pkl"))

        print("Time taken = {:0.3f}s\n".format(time.time() - start))

    def __str__(self):
        return "Saving label map to - {}".format(self.dest)



import os

from .preprocess import preprocess

CONFIG = os.environ["CONFIG"]
VERBOSE = os.environ["VERBOSE"]
FUNCTION = os.environ["FUNCTION"]


def call():
    if FUNCTION == "preprocess":
        preprocess(CONFIG)


if __name__ == '__main__':
    call()



from torchcv.engine import PREPROCESS_ENGINE
from ..config import read_config


def preprocess(config_path: str, engine: dict = PREPROCESS_ENGINE):
    config = read_config(config_path)

    for function in config:
        if function == "paths":
            continue
        if function in engine.keys():
            preprocessor = engine[function](**config[function])
            print(preprocessor)
            preprocessor()
        else:
            raise NotImplementedError(
                "{} has not been implemented. Available preprocessing functions are {}".format(function, engine.keys())
            )



from subprocess import call

from .data.create_images import create_images
from .data.create_configs import create_preprocess_config

create_images()
create_preprocess_config()

CMD = "/tmp/.tox/python/bin/tcv"
CONFIG = "/tmp/torch-cv-test/config/preprocess.yml"


def test_preprocess():
    code = call(["sh", CMD, f"--config={CONFIG}", "--function=preprocess"])
    assert code == 0




import pytest

from torchcv.config import read_config
from .data.create_configs import *

CONFIG_DIR = "/tmp/torch-cv-test/config/"
create_empty_config()
create_none_config()
create_join_config()


@pytest.mark.xfail
def test_raise_exception_when_file_is_empty():
    read_config(os.path.join(CONFIG_DIR, "empty.yml"))


def test_none_constructor():
    config = read_config(os.path.join(CONFIG_DIR, "none.yml"))
    assert config.field0 is None


def test_join_constructor():
    config = read_config(os.path.join(CONFIG_DIR, "join.yml"))
    assert config.field0 == 'a/b/c'



import os

import cv2
import pandas as pd
import torch
from joblib import dump
from torch.utils.data import Dataset, DataLoader

from .base import BasePreprocessingClass


class StatsHelper(Dataset):
    def __init__(self, src: pd.DataFrame, image_col: str = "path"):
        self.images = src[image_col]

    def __len__(self) -> int:
        return len(self.images)

    def __getitem__(self, item):
        image = self.images[item]
        image = cv2.imread(image)
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        image = image.transpose(2, 1, 0)

        return image / 255.


class CalculateStats(BasePreprocessingClass):
    def __init__(self, src: str, dest: str, num_workers: int = 1, batch_size: int = 32, image_col: str = "path"):
        super(CalculateStats, self).__init__()

        df = self._read_csv(src)
        num_workers = self._check_num_threads(num_workers)

        self.dest = dest
        self.dl = DataLoader(StatsHelper(df, image_col), batch_size=batch_size, num_workers=num_workers)

    def __call__(self):
        import time
        start = time.time()

        channel_sum, channel_sum_squared, num_batches = 0., 0., 0.
        for _, data in enumerate(self.dl):
            channel_sum += torch.mean(data, dim=[0, 2, 3])
            channel_sum_squared += torch.mean(data ** 2, dim=[0, 2, 3])
            num_batches += 1

        mean = channel_sum / num_batches
        std = ((channel_sum_squared / num_batches) - mean ** 2) ** 0.5

        dump({"mean": mean, "std": std}, os.path.join(self.dest, "stats.pkl"))
        print("Time taken = {:0.3f}s\n".format(time.time() - start))

    def __str__(self):
        return "Saving the dataset statistics to - {}".format(self.dest)



import glob
import os
from typing import Tuple

import pandas as pd
from sklearn import model_selection

from .base import BasePreprocessingClass


class CreateCsv(BasePreprocessingClass):
    def __init__(self, src: str, dest: str, filetype: str = "jpg", test_split: float = 0.1, val_split: float = 0.1):
        super().__init__()
        self.images, self.categories = self._get_lists(src, filetype)
        self.dest = dest
        self.test_split = test_split
        self.val_split = val_split

    def __call__(self):
        import time
        start = time.time()

        if not os.path.exists(self.dest):
            os.makedirs(self.dest)

        csv_dict = {"path": self.images, "class": self.categories}
        all_ = pd.DataFrame(csv_dict)
        all_.to_csv(os.path.join(self.dest, "all.csv"), index=False)

        train, test = model_selection.train_test_split(all_, test_size=self.test_split, stratify=all_["class"])
        test.to_csv(os.path.join(self.dest, "test.csv"), index=False)

        train, val = model_selection.train_test_split(train, test_size=self.val_split, stratify=train["class"])
        val.to_csv(os.path.join(self.dest, "val.csv"), index=False)
        train.to_csv(os.path.join(self.dest, "train.csv"), index=False)

        print("Time taken = {:0.3f}s\n".format(time.time() - start))

    def __str__(self):
        return "Creating csv(s) at - {}".format(self.dest)



from subprocess import Popen, STDOUT, PIPE

from .data.shell_ouputs import *

CMD = "/tmp/.tox/python/bin/tcv"


def test_help_exit_code():
    call = Popen(["sh", CMD, "-h"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == help_out
    assert code == 1

    call = Popen(["sh", CMD, "--help"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == help_out
    assert code == 1


def test_invalid_option():
    call = Popen(["sh", CMD, "-hx"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == wrong_option_out
    assert code == 1

    call = Popen(["sh", CMD, "--help_wrong"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == wrong_option_out
    assert code == 1


def test_version_exit_code():
    call = Popen(["sh", CMD, "-V"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert code == 1

    call = Popen(["sh", CMD, "--version"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert code == 1


def test_no_args_fail():
    call = Popen(["sh", CMD], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == no_config_out
    assert code == 1


def test_only_config_fail():
    call = Popen(["sh", CMD, "-c=/tmp/config.yml"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == no_function_out
    assert code == 1

    call = Popen(["sh", CMD, "--config=/tmp/config.yml"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == no_function_out
    assert code == 1


def test_only_function_fail():
    call = Popen(["sh", CMD, "-fn=preprocess"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == no_config_out
    assert code == 1

    call = Popen(["sh", CMD, "--function=preprocess"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == no_config_out
    assert code == 1


def test_wrong_function():
    call = Popen(["sh", CMD, "-c=dummy -fn=preprocess_wrong"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == wrong_function_out
    assert code == 1

    call = Popen(["sh", CMD, "-c=dummy --function=preprocess_wrong"], stderr=STDOUT, stdout=PIPE)
    out, code = call.communicate()[0], call.returncode
    assert out == wrong_function_out
    assert code == 1



from torchcv.preprocessing import ResizeImages, CreateCsv, CreateLabelMap, CalculateStats

PREPROCESS_ENGINE = {
    "resize": ResizeImages,
    "csv": CreateCsv,
    "label_map": CreateLabelMap,
    "stats": CalculateStats
}



import os

import yaml

from ._utils import DotDict


def _none_constructor(loader, node):
    return None


def _join_constructor(loader, node):
    fields = loader.construct_sequence(node)
    return os.path.join(*fields)


def read_config(yaml_path: str) -> DotDict:
    yaml_path = os.path.abspath(yaml_path)

    yaml.add_constructor("!none", _none_constructor)
    yaml.add_constructor("!join", _join_constructor)

    with open(yaml_path, "r") as f:
        config = yaml.load(f, Loader=yaml.FullLoader)

    if config is None:
        raise ValueError("Config file cannot be empty")

    return DotDict(config)



help_out = b'This is a CLI tool for preprocessing data, training and evaluating PyTorch image classification models.\n\nUsage - tcv [-h|c|f] [--help|config|function|log]\n    -f  --file         :  Path to file to run code from. Ignored if -fn|--function is passed. -f will be given priority over function\n    -fn --function     :  Flag that determines the function to be done. One of train, test, preprocess. Ignored if -f|--file is passed\n    -c  --config       :  Path to config. Examples provided at - [Insert Link]\n    -v  --verbose      :  [Optional] Verbose Mode\n    -V  --version      :  Version of package\n        --log          :  [Optional] Log output to a file. Default at {pwd}/logs/{timestamp}.log\n        --log_dir      :  [Optional] Path to log directory. Ignored if --log is not passed.\n\n    -h  --help         :  Print help\n'

no_config_out = b'Config file not specified. Please pass the -c=|--config= argument\n\nAvailable options are -\nUsage - tcv [-h|c|f] [--help|config|function|log]\n    -f  --file         :  Path to file to run code from. Ignored if -fn|--function is passed. -f will be given priority over function\n    -f  --function     :  Flag that determines the function to be done. One of train, test, preprocess. Ignored if -f|--file is passed\n    -c  --config       :  Path to config. Examples provided at - [Insert Link]\n    -v  --verbose      :  [Optional] Verbose Mode\n    -V  --version      :  Version of package\n        --log          :  [Optional] Log output to a file. Default at {pwd}/logs/{timestamp}.log\n        --log_dir      :  [Optional] Path to log directory.\n\n    -h  --help         :  Print help\n'

no_function_out = b'Please pass either one of -f=*|--file=* OR -fn=[preprocess|test|train]|--function=[preprocess|test|train]\n\nAvailable options are -\nUsage - tcv [-h|c|f] [--help|config|function|log]\n    -f  --file         :  Path to file to run code from. Ignored if -fn|--function is passed. -f will be given priority over function\n    -f  --function     :  Flag that determines the function to be done. One of train, test, preprocess. Ignored if -f|--file is passed\n    -c  --config       :  Path to config. Examples provided at - [Insert Link]\n    -v  --verbose      :  [Optional] Verbose Mode\n    -V  --version      :  Version of package\n        --log          :  [Optional] Log output to a file. Default at {pwd}/logs/{timestamp}.log\n        --log_dir      :  [Optional] Path to log directory.\n\n    -h  --help         :  Print help\n'

wrong_option_out = b'Invalid option passed to script\n\nAvailable options are -\nUsage - tcv [-h|c|f] [--help|config|function|log]\n    -f  --file         :  Path to file to run code from. Ignored if -fn|--function is passed. -f will be given priority over function\n    -f  --function     :  Flag that determines the function to be done. One of train, test, preprocess. Ignored if -f|--file is passed\n    -c  --config       :  Path to config. Examples provided at - [Insert Link]\n    -v  --verbose      :  [Optional] Verbose Mode\n    -V  --version      :  Version of package\n        --log          :  [Optional] Log output to a file. Default at {pwd}/logs/{timestamp}.log\n        --log_dir      :  [Optional] Path to log directory.\n\n    -h  --help         :  Print help\n'

wrong_function_out = b'Please pass either one of -f=*|--file=* OR -fn=[preprocess|test|train]|--function=[preprocess|test|train]\n\nAvailable options are -\nUsage - tcv [-h|c|f] [--help|config|function|log]\n    -f  --file         :  Path to file to run code from. Ignored if -fn|--function is passed. -f will be given priority over function\n    -f  --function     :  Flag that determines the function to be done. One of train, test, preprocess. Ignored if -f|--file is passed\n    -c  --config       :  Path to config. Examples provided at - [Insert Link]\n    -v  --verbose      :  [Optional] Verbose Mode\n    -V  --version      :  Version of package\n        --log          :  [Optional] Log output to a file. Default at {pwd}/logs/{timestamp}.log\n        --log_dir      :  [Optional] Path to log directory.\n\n    -h  --help         :  Print help\n'



none_config = """
field0: !none
"""

join_config = """
field0: !join ["a", "b", "c"]
"""

preprocess_config = """
paths:
    base: &base "/tmp/torch-cv-test/"
    original_images: !join &orig_images [*base, "original_images/"]
    resized_images: !join &resized_images [*base, "resized_images/"]
    csv_path: !join &csv [*base, "csv/"]
    metadata: !join &metadata [*base, "metadata/"]

resize:
     src: *orig_images
     dest: *resized_images
     size: 224
     num_workers: -1
     batch_size: 32
     filetype: "jpg"

csv:
     src: *resized_images
     dest: *csv
     filetype: "jpg"
     test_split: 0.1
     val_split: 0.1

label_map:
    src: !join [*csv, "all.csv"]
    dest: *metadata
    target: "class"

stats:
    src: !join [*csv, "all.csv"]
    dest: *metadata
    num_workers: 8
    batch_size: 32
    image_col: "path"
"""



import os
import cv2
import numpy as np
import random


default_dir = "/tmp/torch-cv-test/original_images/"


def create_temp_original():
    if not os.path.exists(default_dir):
        os.makedirs(default_dir)


def create_images():
    for i in range(5):
        dest = os.path.join(default_dir, str(i))
        os.makedirs(dest, exist_ok=True)
        for j in range(10):
            h, w = random.randint(500, 1000), random.randint(500, 1000)
            img_array = np.random.randint(0, 255, size=(h, w, 3))
            cv2.imwrite(os.path.join(dest, f"{i}_{j}.jpg"), img_array)


from setuptools import setup, find_packages

with open("README.rst", "r") as fh:
    long_description = fh.read()

dependencies = [
    "torch~=1.7.0",
    "setuptools~=50.3.2",
    "opencv-python~=4.4.0.44",
    "numpy~=1.19.2",
    "PyYAML~=5.3.1",
    "pandas~=1.1.3",
    "joblib~=0.17.0",
    "scikit-learn~=0.23.2"
]

setup(
    name="torch-cv",
    version="0.1.0",
    author="Raghhuveer Jaikanth",
    author_email="raghhuveerj97@gmail.com",
    description="A high level package for Computer Vision with pytorch",
    long_description=long_description,
    long_description_content_type="text/x-rst ",
    url="https://github.com/RJaikanth/torch-cv",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        "Natural Language :: English",
        "Operating System :: Unix"
    ],
    python_requires=">=3.7",
    scripts=["scripts/tcv"],
    install_requires=dependencies
)



class DotDict(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __init__(self, dct):
        super().__init__()

        for key, value in dct.items():
            if hasattr(value, "keys"):
                value = DotDict(value)
            self[key] = value



import os
from typing import AnyStr, Tuple


class BasePreprocessingClass:
    def __init__(self) -> None:
        self.verbose = os.environ["VERBOSE"]

    def __call__(self, *args, **kwargs) -> None:
        raise NotImplementedError("Call function has not been updated yet")

    def __str__(self) -> AnyStr:
        return "Base Preprocessing class"

    def _read_csv(self, path: str):
        import pandas as pd
        return pd.read_csv(path)

    @staticmethod
    def _get_lists(src: str, filetype: str = "jpg") -> Tuple[list, list]:
        import glob
        images, categories = [], []

        for cat in glob.glob(src + "/*"):
            image_list = glob.glob(cat + f"/*.{filetype}")
            cat_list = [os.path.basename(cat)] * len(image_list)

            images += glob.glob(cat + f"/*.{filetype}")
            categories += cat_list

        return images, categories

    @staticmethod
    def _get_max_threads():
        return int(os.popen('grep -c cores /proc/cpuinfo').read())

    def _check_num_threads(self, n: int):
        if n == -1:
            n = self._get_max_threads()
        else:
            assert n <= self._get_max_threads(), "Number of Threads must be less than {}".format(self._get_max_threads())
            assert n > 0, "Number of threads should be at least 1"

        return n



import glob
import os
from typing import List

import cv2
from torch.utils.data import DataLoader, Dataset

from .base import BasePreprocessingClass


class ResizeHelper(Dataset):
    def __init__(self, images: List[str], dest: str, size: int = 224) -> None:
        super(ResizeHelper, self).__init__()

        self.dest = dest
        self.size = size
        self.images = images

    def __getitem__(self, item):
        image = self.images[item]
        category = os.path.basename(os.path.dirname(image))
        os.makedirs(os.path.join(self.dest, category), exist_ok=True)

        self._resize(image, category, size=self.size)

        return image

    def __len__(self):
        return len(self.images)

    def _resize(self, image: str, category: str, size: int = 224):
        image = os.path.abspath(image)
        file_name = os.path.basename(image)
        dest_path = os.path.join(self.dest, category, file_name)

        image = cv2.imread(image)
        image = cv2.resize(image, (size, size))

        cv2.imwrite(dest_path, image)


class ResizeImages(BasePreprocessingClass):
    def __init__(self, src: str, dest: str, size: int = 224, filetype: str = "jpg", num_workers: int = 1,
                 batch_size: int = 32):
        super(ResizeImages, self).__init__()

        num_workers = self._check_num_threads(num_workers)
        images, _ = self._get_lists(src, filetype)

        self.dest = dest
        self.ds = ResizeHelper(images, dest, size)
        self.dl = DataLoader(self.ds, batch_size=batch_size, num_workers=num_workers)

    def __call__(self, ):
        import time
        start = time.time()

        for _ in enumerate(self.dl):
            continue

        print("Time taken = {:0.3f}s\n".format(time.time() - start))

    def __str__(self):
        return "Resizing images to size - {} at {}".format(self.ds.size, self.dest)
